from unittest import mock


@mock.patch("pysisu.PySisu.fetch_sisu_api")
def test_time_compare_csv(
    mock_pysisu, result_time_compare, pysisu_client, pysisu_output_snapshot
):
    mock_pysisu.return_value = result_time_compare.to_dict()
    pysisu_output_snapshot.assert_match(
        pysisu_client.get_results(1).to_csv(), "time_compare.csv"
    )


@mock.patch("pysisu.PySisu.fetch_sisu_api")
def test_group_compare_csv(
    mock_pysisu, result_group_compare, pysisu_client, pysisu_output_snapshot
):
    mock_pysisu.return_value = result_group_compare.to_dict()
    pysisu_output_snapshot.assert_match(
        pysisu_client.get_results(1).to_csv(), "group_compare.csv"
    )


@mock.patch("pysisu.PySisu.fetch_sisu_api")
def test_general_performance_csv(
    mock_pysisu,
    result_general_performance,
    pysisu_client,
    pysisu_output_snapshot,
):
    mock_pysisu.return_value = result_general_performance.to_dict()
    pysisu_output_snapshot.assert_match(
        pysisu_client.get_results(1).to_csv(), "general_performance.csv"
    )


@mock.patch("pysisu.PySisu.fetch_sisu_api")
def test_trend_detection_csv(
    mock_fetch, result_trend, pysisu_client, pysisu_output_snapshot
):
    mock_fetch.return_value = result_trend.to_dict()
    pysisu_output_snapshot.assert_match(
        pysisu_client.get_results(1).to_csv(), "trend_detection.csv"
    )



#
# Copyright 2022 Sisu Data, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from dataclasses import dataclass
from typing import Dict, List, Optional, Tuple, Union
from .formats import HeaderColumn, Row, Table
from .proto.sisu.v1.api import (
    AnalysisRunResultsResponse,
    Factor,
    Value,
    KeyDriverAnalysisResultGroupComparison,
    KeyDriverAnalysisResultSegment,
    KeyDriverAnalysisResultTimeComparison,
    TrendAnalysisResultSegment,
    TrendAnalysisResultTrend,
)
import datetime
import betterproto


@dataclass
class LatestAnalysisResultTable(Table):
    pass


@dataclass
class KDARow(Row):
    subgroup_id: int
    confidence: str
    factor_0_dimension: str
    factor_0_value: any
    factor_1_dimension: str
    factor_1_value: any
    factor_2_dimension: str
    factor_2_value: any
    impact: float


@dataclass
class TimeCompareRow(KDARow):
    previous_period_size: float
    recent_period_size: float
    previous_period_value: float
    recent_period_value: float
    previous_period_start_date_inclusive: datetime.datetime
    previous_period_end_date_inclusive: datetime.datetime
    recent_period_start_date_inclusive: datetime.datetime
    recent_period_end_date_inclusive: datetime.datetime


@dataclass
class GroupCompareRow(KDARow):
    group_a_size: float
    group_b_size: float
    group_a_value: float
    group_b_value: float
    group_a_name: str
    group_b_name: str


@dataclass
class GeneralPerformanceRow(KDARow):
    size: float
    value: float


@dataclass
class TrendRow(Row):
    subgroup_id: int
    factor_0_dimension: str
    factor_0_value: any
    factor_1_dimension: str
    factor_1_value: any
    factor_2_dimension: str
    factor_2_value: any
    impact: float
    start_date_inclusive: datetime.datetime
    end_date_inclusive: datetime.datetime
    intercept: Optional[float]
    slope: Optional[float]
    size: Optional[float]


class safelist(list):
    # taken from https://stackoverflow.com/questions/5125619/why-doesnt-list-have-safe-get-method-like-dictionary
    def get(self, index, default=None):
        try:
            return self.__getitem__(index)
        except IndexError:
            return default


def get_factor_value(
    factor_value: Value,
    round_to_decimal_place: int = 2
) -> Union[str, int, float, bool, datetime.datetime]:
    _val_type, value = betterproto.which_one_of(factor_value, "value_type")
    return get_rounded_value(value, round_to_decimal_place)


@dataclass
class FactorDimVal:
    dimension: Optional[str]
    value: Optional[any]


def get_factor(dimension: str, factor: Factor, round_to_decimal_place: int = 2) -> FactorDimVal:
    factor_type, _factor_data = betterproto.which_one_of(factor, "factor_type")
    if factor_type == "value":
        return FactorDimVal(dimension, get_factor_value(factor.value, round_to_decimal_place))
    elif factor_type == "bin":
        return FactorDimVal(
            dimension,
            f"{get_rounded_value(factor.bin.lower_bound , round_to_decimal_place)}-{get_rounded_value(factor.bin.upper_bound , round_to_decimal_place)} ({get_rounded_value(factor.bin.lower_bound_percentile , round_to_decimal_place)}-{get_rounded_value(factor.bin.upper_bound_percentile , round_to_decimal_place)} percentile)",
        )
    elif factor_type == "keyword":
        return FactorDimVal(dimension, factor.keyword.keyword)
    else:
        raise ValueError("invalid factor")


def get_factors(
    factors: Dict[str, Factor], round_to_decimal_place: int = 2
) -> Tuple[FactorDimVal, FactorDimVal, FactorDimVal]:
    converted_factors = safelist(
        [get_factor(dim, fact, round_to_decimal_place) for dim, fact in factors.items()]
    )    
    empty_factor = FactorDimVal(None, None)
    return (
        converted_factors.get(0),
        converted_factors.get(1, empty_factor),
        converted_factors.get(2, empty_factor),
    )


def build_header_from_row(rows: List[Row]) -> List[HeaderColumn]:
    if not rows:
        return []
    row = rows[0]
    return [
        HeaderColumn(
            var, str if getattr(row, var) is None else type(getattr(row, var))
        )
        for var in vars(row).keys()
    ]


def get_rows_time_comparision(
    subgroups: List[KeyDriverAnalysisResultSegment],
    time_comparision: KeyDriverAnalysisResultTimeComparison,
    round_to_decimal_place: int = 2
) -> List[Row]:
    rows = []
    for subgroup in subgroups:
        factor_0, factor_1, factor_2 = get_factors(subgroup.factors, round_to_decimal_place)

        r = TimeCompareRow(
            subgroup_id=subgroup.id,
            confidence=subgroup.confidence.name,
            factor_0_dimension=factor_0.dimension,
            factor_0_value=factor_0.value,
            factor_1_dimension=factor_1.dimension,
            factor_1_value=factor_1.value,
            factor_2_dimension=factor_2.dimension,
            factor_2_value=factor_2.value,
            previous_period_size=get_rounded_value(subgroup.time_comparison.previous_period_size, round_to_decimal_place),
            recent_period_size=get_rounded_value(subgroup.time_comparison.recent_period_size, round_to_decimal_place),
            previous_period_value=get_rounded_value(subgroup.time_comparison.previous_period_value, round_to_decimal_place),
            recent_period_value=get_rounded_value(subgroup.time_comparison.recent_period_value, round_to_decimal_place),
            previous_period_start_date_inclusive=time_comparision.previous_period.start_date_inclusive,
            previous_period_end_date_inclusive=time_comparision.previous_period.end_date_inclusive,
            recent_period_start_date_inclusive=time_comparision.recent_period.start_date_inclusive,
            recent_period_end_date_inclusive=time_comparision.recent_period.end_date_inclusive,
            impact=get_rounded_value(subgroup.impact, round_to_decimal_place),
        )
        rows.append(r)

    return rows


def get_rows_group_comparision(
    subgroups: List[KeyDriverAnalysisResultSegment],
    group_comparision: KeyDriverAnalysisResultGroupComparison,
    round_to_decimal_place: int = 2
) -> List[Row]:
    rows = []
    for subgroup in subgroups:
        factor_0, factor_1, factor_2 = get_factors(subgroup.factors, round_to_decimal_place)

        r = GroupCompareRow(
            subgroup_id=subgroup.id,
            confidence=subgroup.confidence.name,
            factor_0_dimension=factor_0.dimension,
            factor_0_value=factor_0.value, 
            factor_1_dimension=factor_1.dimension,
            factor_1_value=factor_1.value, 
            factor_2_dimension=factor_2.dimension,
            factor_2_value=factor_2.value,
            group_a_size=get_rounded_value(subgroup.group_comparison.group_a_size, round_to_decimal_place),
            group_b_size=get_rounded_value(subgroup.group_comparison.group_b_size, round_to_decimal_place),
            group_a_value=get_rounded_value(subgroup.group_comparison.group_a_value, round_to_decimal_place),
            group_b_value=get_rounded_value(subgroup.group_comparison.group_b_value, round_to_decimal_place),
            group_a_name=group_comparision.group_a.name,
            group_b_name=group_comparision.group_b.name,
            impact=get_rounded_value(subgroup.impact, round_to_decimal_place),
        )
        rows.append(r)

    return rows


def get_rows_general_performance(
    subgroups: List[KeyDriverAnalysisResultSegment],
    round_to_decimal_place: int = 2
) -> List[Row]:
    rows = []
    for subgroup in subgroups:
        factor_0, factor_1, factor_2 = get_factors(subgroup.factors, round_to_decimal_place)

        r = GeneralPerformanceRow(
            subgroup_id=subgroup.id,
            confidence=subgroup.confidence.name,
            factor_0_dimension=factor_0.dimension,
            factor_0_value=factor_0.value,
            factor_1_dimension=factor_1.dimension,
            factor_1_value=factor_1.value,
            factor_2_dimension=factor_2.dimension,
            factor_2_value=factor_2.value,
            size=get_rounded_value(subgroup.general_performance.size, round_to_decimal_place),
            value=get_rounded_value(subgroup.general_performance.value, round_to_decimal_place),
            impact=get_rounded_value(subgroup.impact, round_to_decimal_place),
        )
        rows.append(r)

    return rows


def get_rows_trend(
    subgroups: List[TrendAnalysisResultSegment],
    overall_trends: List[TrendAnalysisResultTrend],
    round_to_decimal_place: int = 2
) -> List[Row]:
    rows = []

    for otrend in overall_trends:
        r = TrendRow(
            subgroup_id=None,
            factor_0_dimension=None,
            factor_0_value=None,
            factor_1_dimension=None,
            factor_1_value=None,
            factor_2_dimension=None,
            factor_2_value=None,
            impact=get_rounded_value(otrend.trend, round_to_decimal_place),
            start_date_inclusive=otrend.time_range.start_date_inclusive,
            end_date_inclusive=otrend.time_range.end_date_inclusive,
            intercept=get_rounded_value(otrend.intercept, round_to_decimal_place) ,
            slope=get_rounded_value(otrend.trend, round_to_decimal_place) ,
            size=get_rounded_value(otrend.size, round_to_decimal_place) ,
        )
        rows.append(r)

    for subgroup in subgroups:
        factor_0, factor_1, factor_2 = get_factors(subgroup.factors, round_to_decimal_place)

        if factor_1.dimension is not None:
            if factor_2.dimension is not None:
                factor_0, factor_1, factor_2 = sorted(
                    (factor_0, factor_1, factor_2),
                    key=lambda factor: factor.dimension,
                )
            else:
                factor_0, factor_1 = sorted(
                    (factor_0, factor_1), key=lambda factor: factor.dimension
                )

        for trend in subgroup.trends:
            r = TrendRow(
                subgroup_id=subgroup.id,
                factor_0_dimension=factor_0.dimension,
                factor_0_value=factor_0.value,
                factor_1_dimension=factor_1.dimension,
                factor_1_value=factor_1.value,
                factor_2_dimension=factor_2.dimension,
                factor_2_value=factor_2.value,
                impact=get_rounded_value(subgroup.impact, round_to_decimal_place),
                start_date_inclusive=trend.time_range.start_date_inclusive,
                end_date_inclusive=trend.time_range.end_date_inclusive,
                intercept=get_rounded_value(trend.intercept, round_to_decimal_place) ,
                slope=get_rounded_value(trend.trend, round_to_decimal_place) ,
                size=get_rounded_value(trend.size, round_to_decimal_place) ,
            )
            rows.append(r)

    return rows
def get_rounded_value(value, round_to_decimal_place):
    return round(value, round_to_decimal_place) if type(value) is float else value


def _get_rows(result: AnalysisRunResultsResponse, round_to_decimal_place: int = 2) -> List[Row]:
    # Under the assumption that an analysis can't be
    # more than one of [KDA, TD]
    if result.analysis_result.key_driver_analysis_result:
        analysis_result = result.analysis_result.key_driver_analysis_result
        subgroups = analysis_result.segments

        if analysis_result.time_comparison:
            return get_rows_time_comparision(
                subgroups, analysis_result.time_comparison, round_to_decimal_place
            )
        elif analysis_result.group_comparison:
            return get_rows_group_comparision(
                subgroups, analysis_result.group_comparison, round_to_decimal_place
            )
        elif analysis_result.general_performance._serialized_on_wire:
            return get_rows_general_performance(subgroups, round_to_decimal_place)
        else:
            raise ValueError("Invalid key_driver_analysis_result")

    elif result.analysis_result.trend_analysis_result:
        analysis_result = result.analysis_result.trend_analysis_result
        subgroups = analysis_result.segments

        return get_rows_trend(subgroups, analysis_result.overall_trends, round_to_decimal_place)

    else:
        raise ValueError("Invalid analysis_result")


def to_table(result: AnalysisRunResultsResponse, round_to_decimal_place: int = 2) -> LatestAnalysisResultTable:
    rows = _get_rows(result, round_to_decimal_place)
    return LatestAnalysisResultTable(build_header_from_row(rows), rows)



from unittest import mock

import pytest
from pysisu import PySisu


@mock.patch("pysisu.pysisu_class.PYSISU_VERSION", "1.1.0")
@mock.patch("requests.Response.json", lambda x: {"info": {"version": "2.0.0"}})
def test_major_version_exception():
    with pytest.raises(Exception, match="breaking changes"):
        PySisu("test")


@mock.patch("pysisu.pysisu_class.PYSISU_VERSION", "0.1.0")
@mock.patch("requests.Response.json", lambda x: {"info": {"version": "0.2.0"}})
def test_experimental_minor_version_exception():
    with pytest.raises(Exception, match="breaking changes"):
        PySisu("test")


@mock.patch("pysisu.pysisu_class.PYSISU_VERSION", "1.2.3")
@mock.patch("requests.Response.json", lambda x: {"info": {"version": "1.3.0"}})
def test_minor_version_warning():
    with pytest.warns(UserWarning, match="Pysisu has been updated from"):
        PySisu("test")


@mock.patch("requests.Response.json", lambda x: {})
def test_pypi_failure_warning():
    with pytest.warns(UserWarning, match="Unable to verify"):
        PySisu("test")


@mock.patch("pysisu.pysisu_class.PYSISU_VERSION", "0.0.0")
def test_compatibility_check_skipped_in_pytest(pysisu_client):
    with pytest.raises(Exception, match="breaking changes"):
        PySisu("test")

    # mocked fixture compatibility check will not raise an Exception
    assert pysisu_client



#
# Copyright 2022 Sisu Data, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import warnings
from http import HTTPStatus
from typing import Optional, Union

import requests

from .formats import LatestAnalysisResultsFormats, Table
from .latest_analysis_result import to_table
from .proto.sisu.v1.api import (
    AnalysesListResponse,
    AnalysisDimensionsListResponse,
    AnalysisRunResultsResponse,
    CreateDataSetRequestDataset,
    DataSetsResponse,
    DataSourceListResponse,
    DuplicateAnalysisResponse,
    GetAnalysisFiltersResponse,
    GetProjectsAnalysesListResponse,
    GetProjectsListResponseListProjectResponse,
    GetSegmentDataResponse,
    MetricsListResponse,
    ModifyAnalysisRequestAnalysis,
    ModifyAnalysisResponse,
    SetAnalysisFiltersRequest,
    UpdateAnalysisRequestAnalysis,
    UpdateAnalysisResponse,
    WaterfallAnalysisResponse,
)
from .query_helpers import build_url, pathjoin, semver_parse
from .version import __version__ as PYSISU_VERSION


class PySisuBaseException(Exception):
    pass


class PySisuInvalidReturnedPaginationHintsFromServer(PySisuBaseException):
    pass


class PySisuInvalidResponseFromServer(PySisuBaseException):
    pass


class PySisuDeprecatedVersionException(PySisuBaseException):
    pass


class PySisu:
    """
    Allows the ability to fetch/send commands to the sisu_api.
    """

    _url: str
    _api_key: str

    def __init__(self, api_key: str, url: str = "https://vip.sisu.ai") -> None:
        self._check_compatibility()
        self._url = url
        self._api_key = api_key

    def set_url(self, url: str) -> "PySisu":
        self._url = url
        return self

    def set_api_key(self, api_key: str) -> "PySisu":
        self._api_key = api_key
        return self

    def _auto_paginate(
        self,
        analysis_id: int,
        params: dict,
        result: AnalysisRunResultsResponse,
        **kwargs,
    ) -> AnalysisRunResultsResponse:
        """
        Fetches the rest of the results if there is more results to fetch.

        Respects the limit parameter and only fetches a maximum of params['limit'] that is left.
        """
        if not result.pagination_hints.has_more:
            return result

        kda_result = result.analysis_result.key_driver_analysis_result
        subgroups = kda_result.segments

        kwargs = {**params, **kwargs}  # fold together so we just use 1 dict
        if kwargs.get("limit"):
            kwargs["limit"] -= len(subgroups)

        if kwargs.get("limit", 1) <= 0 or len(subgroups) == 0:
            return result

        next_cursor = int(subgroups[-1].id)
        if result.pagination_hints.has_more:
            if result.pagination_hints.next_starting_cursor is None:
                raise PySisuInvalidReturnedPaginationHintsFromServer(
                    "There is more to fetch, however the next_starting cursor is none."
                )

        kwargs["starting_after"] = next_cursor

        next_page = self.get_results(
            analysis_id, format=LatestAnalysisResultsFormats.PROTO, **kwargs
        )
        kda_result.segments = (
            subgroups
            + next_page.analysis_result.key_driver_analysis_result.segments
        )
        return result

    def _call_sisu_api(
        self, url_path: int, request_method="get", json=None
    ) -> dict:
        headers = {
            "Authorization": self._api_key,
            "User-Agent": f"PySisu/{PYSISU_VERSION}",
        }
        r = requests.request(
            request_method, url_path, headers=headers, json=json
        )
        if r.status_code != HTTPStatus.OK:
            raise PySisuInvalidResponseFromServer(
                "Result did not complete", r.content
            )
        return r.json()

    def _check_compatibility(self):
        r = requests.get("https://pypi.python.org/pypi/pysisu/json")
        try:
            latest_version = r.json()["info"]["version"]
        except (requests.RequestException, KeyError):
            warnings.warn(
                "Unable to verify current pysisu version with PyPI; get "
                "latest version with 'pip install -U pysisu'"
            )
            return

        local_major, local_minor, local_patch = semver_parse(PYSISU_VERSION)
        latest_major, latest_minor, latest_patch = semver_parse(latest_version)

        if local_major < latest_major or (
            local_major == 0 and local_minor < latest_minor
        ):
            raise PySisuDeprecatedVersionException(
                "Pysisu has been updated with breaking changes from "
                f"{PYSISU_VERSION} to {latest_version}; get latest version "
                "with 'pip install -U pysisu'"
            )
        if (local_minor, local_patch) < (latest_minor, latest_patch):
            warnings.warn(
                f"Pysisu has been updated from {PYSISU_VERSION} to "
                f"{latest_version}; get latest version with 'pip install -U "
                "pysisu'"
            )

    def fetch_sisu_api(
        self, analysis_id: int, params: dict = {}, **kwargs
    ) -> dict:
        path = ["api/v1/analyses/", str(analysis_id), "runs/latest"]
        url_path = build_url(self._url, pathjoin(*path), {**params, **kwargs})
        return self._call_sisu_api(url_path)

    def run(self, analysis_id: int, **kwargs):
        path = ["api/v1/analyses/", str(analysis_id), "run"]
        url_path = build_url(self._url, pathjoin(*path), kwargs)
        self._call_sisu_api(url_path, request_method="POST")

    def analyses(self, **kwargs) -> AnalysesListResponse:
        path = ["api/v1/analyses"]
        url_path = build_url(self._url, pathjoin(*path), kwargs)
        return AnalysesListResponse().from_dict(self._call_sisu_api(url_path))

    def get_results(
        self,
        analysis_id: int,
        params: dict = {"confidence_gte": "LOW"},
        auto_paginate: bool = True,
        format: LatestAnalysisResultsFormats = LatestAnalysisResultsFormats.TABLE,
        round_to_decimal_place: int = 2,
        **kwargs,
    ) -> Union[AnalysisRunResultsResponse, Table]:
        assert type(params) is dict
        assert type(kwargs) is dict
        kwargs = {**params, **kwargs}

        result = AnalysisRunResultsResponse().from_dict(
            self.fetch_sisu_api(analysis_id, **kwargs)
        )
        if auto_paginate:
            result = self._auto_paginate(analysis_id, kwargs, result)

        if format == LatestAnalysisResultsFormats.TABLE:
            print("here")
            return to_table(result, round_to_decimal_place)
        else:
            return result

    def metrics(self, **kwargs) -> MetricsListResponse:
        path = ["api/v1/metrics"]
        url_path = build_url(self._url, pathjoin(*path), kwargs)
        return MetricsListResponse().from_dict(self._call_sisu_api(url_path))

    def data_sources(self, **kwargs) -> DataSourceListResponse:
        path = ["api/v1/data_sources"]
        url_path = build_url(self._url, pathjoin(*path), kwargs)
        return DataSourceListResponse().from_dict(
            self._call_sisu_api(url_path)
        )

    def datasets(self, **kwargs) -> DataSetsResponse:
        path = ["api/v1/datasets"]
        url_path = build_url(self._url, pathjoin(*path), kwargs)
        return DataSetsResponse().from_dict(self._call_sisu_api(url_path))

    def get_filters(
        self, analysis_id: int, **kwargs
    ) -> GetAnalysisFiltersResponse:
        path = [f"api/v1/analyses/{analysis_id}/filters"]
        url_path = build_url(self._url, pathjoin(*path), kwargs)
        return GetAnalysisFiltersResponse().from_dict(
            self._call_sisu_api(url_path)
        )

    def set_filters(
        self, analysis_id: int, expr: SetAnalysisFiltersRequest, **kwargs
    ):
        path = [f"api/v1/analyses/{analysis_id}/filters"]
        url_path = build_url(self._url, pathjoin(*path), kwargs)
        expr = (
            expr.to_dict()
            if isinstance(expr, SetAnalysisFiltersRequest)
            else expr
        )
        return self._call_sisu_api(url_path, request_method="PUT", json=expr)

    def duplicate_analysis(
        self, analysis_id: int, name: Optional[str] = None, **kwargs
    ) -> DuplicateAnalysisResponse:
        path = [f"api/v1/analyses/{analysis_id}/duplicate"]
        url_path = build_url(self._url, pathjoin(*path), kwargs)
        json = {"name": name} if name else None
        return DuplicateAnalysisResponse().from_dict(
            self._call_sisu_api(url_path, request_method="POST", json=json)
        )

    def get_factor_data(self, segment_id: int) -> GetSegmentDataResponse:
        path = [f"api/v1/segments/{segment_id}/data_query"]
        url_path = build_url(self._url, pathjoin(*path), {})
        return GetSegmentDataResponse().from_dict(
            self._call_sisu_api(url_path, request_method="GET")
        )

    def create_dataset(
        self, data_source_id: int, body: CreateDataSetRequestDataset
    ) -> GetSegmentDataResponse:
        path = [f"api/v1/data_sources/{data_source_id}/datasets"]
        url_path = build_url(self._url, pathjoin(*path), {})
        return GetSegmentDataResponse().from_dict(
            self._call_sisu_api(
                url_path, request_method="POST", json=body.to_dict()
            )
        )

    def get_projects(self) -> GetProjectsListResponseListProjectResponse:
        path = ["api/v1/projects"]
        url_path = build_url(self._url, pathjoin(*path), {})
        return GetProjectsListResponseListProjectResponse().from_dict(
            self._call_sisu_api(url_path, request_method="GET")
        )

    def get_project_analyses_list(
        self, project_id
    ) -> GetProjectsAnalysesListResponse:
        path = [f"api/v1/projects/{project_id}/analyses"]
        url_path = build_url(self._url, pathjoin(*path), {})
        return GetProjectsAnalysesListResponse().from_dict(
            self._call_sisu_api(url_path, request_method="GET")
        )

    def modify_analysis(
        self,
        analysis_id: int,
        modify_analysis_req: ModifyAnalysisRequestAnalysis,
    ) -> ModifyAnalysisResponse:
        path = [f"api/v1/analyses/{analysis_id}"]
        url_path = build_url(self._url, pathjoin(*path), {})
        return ModifyAnalysisResponse().from_dict(
            self._call_sisu_api(
                url_path,
                request_method="PATCH",
                json=modify_analysis_req.to_dict(),
            )
        )

    def update_analysis(
        self,
        analysis_id: int,
        modify_analysis_req: UpdateAnalysisRequestAnalysis,
    ) -> UpdateAnalysisResponse:
        path = [f"api/v1/analyses/{analysis_id}"]
        url_path = build_url(self._url, pathjoin(*path), {})
        return UpdateAnalysisResponse().from_dict(
            self._call_sisu_api(
                url_path,
                request_method="PUT",
                json=modify_analysis_req.to_dict(),
            )
        )

    def get_analysis_dimensions_list(
        self, analysis_id
    ) -> AnalysisDimensionsListResponse:
        path = [f"api/v1/analyses/{analysis_id}/dimensions"]
        url_path = build_url(self._url, pathjoin(*path), {})
        return AnalysisDimensionsListResponse().from_dict(
            self._call_sisu_api(url_path, request_method="GET")
        )
    def get_analysis_waterfall(self, analysis_id) -> WaterfallAnalysisResponse:
        path = [f"api/v1/analyses/{analysis_id}/waterfall"]
        url_path = build_url(self._url, pathjoin(*path), {})
        return WaterfallAnalysisResponse().from_dict(
            self._call_sisu_api(url_path, request_method="GET")
        )



from unittest import mock

from pysisu.pysisu_class import PySisu
from pysisu.formats import LatestAnalysisResultsFormats
from pysisu.proto.sisu.v1.api import AnalysisRunResultsResponse
from .helpers_for_tests import SNAPSHOT_DICT


@mock.patch("pysisu.pysisu_class.PySisu.get_results")
def test_pagination_starting_after_gets_updated(
    get_results: mock.MagicMock,
    result_time_compare: AnalysisRunResultsResponse,
    result_has_more: AnalysisRunResultsResponse,
    pysisu_client: PySisu,
):
    get_results.return_value = result_time_compare
    pysisu_client._auto_paginate(1, {}, result_has_more)

    # Make sure limit is truncated at MAX_LIMIT
    assert "limit" not in get_results.call_args.kwargs

    # Make sure the next starting_after was marked at the correct cursor from the pagination hints
    assert (
        get_results.call_args.kwargs["starting_after"]
        == result_has_more.analysis_result.key_driver_analysis_result.segments[
            -1
        ].id
    )


@mock.patch("pysisu.PySisu.get_results")
def test_pagination_limit_works(
    get_results: mock.MagicMock,
    result_time_compare: AnalysisRunResultsResponse,
    result_has_more: AnalysisRunResultsResponse,
    pysisu_client: PySisu,
):
    get_results.return_value = result_time_compare
    pysisu_client._auto_paginate(1, {}, result_has_more, limit=100)

    assert get_results.call_args.kwargs["limit"] == 100 - len(
        result_time_compare.analysis_result.key_driver_analysis_result.segments
    )
    assert (
        get_results.call_args.kwargs["starting_after"]
        == result_time_compare.analysis_result.key_driver_analysis_result.segments[
            -1
        ].id
    )


@mock.patch("pysisu.PySisu.get_results")
def test_pagination_results_make_sense(
    get_results: mock.MagicMock,
    result_time_compare: AnalysisRunResultsResponse,
    result_has_more: AnalysisRunResultsResponse,
    pysisu_client: PySisu,
    pysisu_input_snapshot,
):
    get_results.return_value = result_time_compare
    result = pysisu_client._auto_paginate(1, {}, result_has_more, limit=100)
    pysisu_input_snapshot.assert_match(
        result.to_json(indent=4), SNAPSHOT_DICT["time compare has more"]
    )


@mock.patch("pysisu.PySisu.fetch_sisu_api")
def test_pagination_results_using_customer_facing_functions(
    fetch_sisu_api: mock.MagicMock,
    result_time_compare: AnalysisRunResultsResponse,
    result_has_more: AnalysisRunResultsResponse,
    pysisu_client: PySisu,
    pysisu_input_snapshot,
):
    fetch_sisu_api.side_effect = iter(
        [result_has_more.to_dict(), result_time_compare.to_dict()]
    )
    result = pysisu_client.get_results(
        1, {"limit": 100}, format=LatestAnalysisResultsFormats.PROTO
    )
    pysisu_input_snapshot.assert_match(
        result.to_json(indent=4),
        SNAPSHOT_DICT["time compare has more doubled"],
    )

    assert fetch_sisu_api.call_args.kwargs["limit"] == 100 - len(
        result_time_compare.analysis_result.key_driver_analysis_result.segments
    )
    assert (
        fetch_sisu_api.call_args.kwargs["starting_after"]
        == result_has_more.analysis_result.key_driver_analysis_result.segments[
            -1
        ].id
    )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: google/api/annotations.proto, google/api/http.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class Http(betterproto.Message):
    """
    Defines the HTTP configuration for an API service. It contains a list of
    [HttpRule][google.api.HttpRule], each specifying the mapping of an RPC
    method to one or more HTTP REST API methods.
    """

    rules: List["HttpRule"] = betterproto.message_field(1)
    """
    A list of HTTP configuration rules that apply to individual API methods.
    **NOTE:** All service configuration rules follow "last one wins" order.
    """

    fully_decode_reserved_expansion: bool = betterproto.bool_field(2)
    """
    When set to true, URL path parameters will be fully URI-decoded except in
    cases of single segment matches in reserved expansion, where "%2F" will be
    left encoded. The default behavior is to not decode RFC 6570 reserved
    characters in multi segment matches.
    """


@dataclass(eq=False, repr=False)
class HttpRule(betterproto.Message):
    """
    # gRPC Transcoding gRPC Transcoding is a feature for mapping between a gRPC
    method and one or more HTTP REST endpoints. It allows developers to build a
    single API service that supports both gRPC APIs and REST APIs. Many
    systems, including [Google APIs](https://github.com/googleapis/googleapis),
    [Cloud Endpoints](https://cloud.google.com/endpoints), [gRPC
    Gateway](https://github.com/grpc-ecosystem/grpc-gateway), and
    [Envoy](https://github.com/envoyproxy/envoy) proxy support this feature and
    use it for large scale production services. `HttpRule` defines the schema
    of the gRPC/REST mapping. The mapping specifies how different portions of
    the gRPC request message are mapped to the URL path, URL query parameters,
    and HTTP request body. It also controls how the gRPC response message is
    mapped to the HTTP response body. `HttpRule` is typically specified as an
    `google.api.http` annotation on the gRPC method. Each mapping specifies a
    URL path template and an HTTP method. The path template may refer to one or
    more fields in the gRPC request message, as long as each field is a non-
    repeated field with a primitive (non-message) type. The path template
    controls how fields of the request message are mapped to the URL path.
    Example:     service Messaging {       rpc GetMessage(GetMessageRequest)
    returns (Message) {         option (google.api.http) = {             get:
    "/v1/{name=messages/*}"         };       }     }     message
    GetMessageRequest {       string name = 1; // Mapped to URL path.     }
    message Message {       string text = 1; // The resource content.     }
    This enables an HTTP REST to gRPC mapping as below: HTTP | gRPC -----|-----
    `GET /v1/messages/123456`  | `GetMessage(name: "messages/123456")` Any
    fields in the request message which are not bound by the path template
    automatically become HTTP query parameters if there is no HTTP request
    body. For example:     service Messaging {       rpc
    GetMessage(GetMessageRequest) returns (Message) {         option
    (google.api.http) = {             get:"/v1/messages/{message_id}"
    };       }     }     message GetMessageRequest {       message SubMessage {
    string subfield = 1;       }       string message_id = 1; // Mapped to URL
    path.       int64 revision = 2;    // Mapped to URL query parameter
    `revision`.       SubMessage sub = 3;    // Mapped to URL query parameter
    `sub.subfield`.     } This enables a HTTP JSON to RPC mapping as below:
    HTTP | gRPC -----|----- `GET
    /v1/messages/123456?revision=2&sub.subfield=foo` | `GetMessage(message_id:
    "123456" revision: 2 sub: SubMessage(subfield: "foo"))` Note that fields
    which are mapped to URL query parameters must have a primitive type or a
    repeated primitive type or a non-repeated message type. In the case of a
    repeated type, the parameter can be repeated in the URL as
    `...?param=A&param=B`. In the case of a message type, each field of the
    message is mapped to a separate parameter, such as
    `...?foo.a=A&foo.b=B&foo.c=C`. For HTTP methods that allow a request body,
    the `body` field specifies the mapping. Consider a REST update method on
    the message resource collection:     service Messaging {       rpc
    UpdateMessage(UpdateMessageRequest) returns (Message) {         option
    (google.api.http) = {           patch: "/v1/messages/{message_id}"
    body: "message"         };       }     }     message UpdateMessageRequest {
    string message_id = 1; // mapped to the URL       Message message = 2;   //
    mapped to the body     } The following HTTP JSON to RPC mapping is enabled,
    where the representation of the JSON in the request body is determined by
    protos JSON encoding: HTTP | gRPC -----|----- `PATCH /v1/messages/123456 {
    "text": "Hi!" }` | `UpdateMessage(message_id: "123456" message { text:
    "Hi!" })` The special name `*` can be used in the body mapping to define
    that every field not bound by the path template should be mapped to the
    request body.  This enables the following alternative definition of the
    update method:     service Messaging {       rpc UpdateMessage(Message)
    returns (Message) {         option (google.api.http) = {           patch:
    "/v1/messages/{message_id}"           body: "*"         };       }     }
    message Message {       string message_id = 1;       string text = 2;     }
    The following HTTP JSON to RPC mapping is enabled: HTTP | gRPC -----|-----
    `PATCH /v1/messages/123456 { "text": "Hi!" }` | `UpdateMessage(message_id:
    "123456" text: "Hi!")` Note that when using `*` in the body mapping, it is
    not possible to have HTTP parameters, as all fields not bound by the path
    end in the body. This makes this option more rarely used in practice when
    defining REST APIs. The common usage of `*` is in custom methods which
    don't use the URL at all for transferring data. It is possible to define
    multiple HTTP methods for one RPC by using the `additional_bindings`
    option. Example:     service Messaging {       rpc
    GetMessage(GetMessageRequest) returns (Message) {         option
    (google.api.http) = {           get: "/v1/messages/{message_id}"
    additional_bindings {             get:
    "/v1/users/{user_id}/messages/{message_id}"           }         };       }
    }     message GetMessageRequest {       string message_id = 1;       string
    user_id = 2;     } This enables the following two alternative HTTP JSON to
    RPC mappings: HTTP | gRPC -----|----- `GET /v1/messages/123456` |
    `GetMessage(message_id: "123456")` `GET /v1/users/me/messages/123456` |
    `GetMessage(user_id: "me" message_id: "123456")` ## Rules for HTTP mapping
    1. Leaf request fields (recursive expansion nested messages in the request
    message) are classified into three categories:    - Fields referred by the
    path template. They are passed via the URL path.    - Fields referred by
    the [HttpRule.body][google.api.HttpRule.body]. They are passed via the HTTP
    request body.    - All other fields are passed via the URL query
    parameters, and the      parameter name is the field path in the request
    message. A repeated      field can be represented as multiple query
    parameters under the same      name.  2. If
    [HttpRule.body][google.api.HttpRule.body] is "*", there is no URL query
    parameter, all fields     are passed via URL path and HTTP request body.
    3. If [HttpRule.body][google.api.HttpRule.body] is omitted, there is no
    HTTP request body, all     fields are passed via URL path and URL query
    parameters. ### Path template syntax     Template = "/" Segments [ Verb ] ;
    Segments = Segment { "/" Segment } ;     Segment  = "*" | "**" | LITERAL |
    Variable ;     Variable = "{" FieldPath [ "=" Segments ] "}" ;
    FieldPath = IDENT { "." IDENT } ;     Verb     = ":" LITERAL ; The syntax
    `*` matches a single URL path segment. The syntax `**` matches zero or more
    URL path segments, which must be the last part of the URL path except the
    `Verb`. The syntax `Variable` matches part of the URL path as specified by
    its template. A variable template must not contain other variables. If a
    variable matches a single path segment, its template may be omitted, e.g.
    `{var}` is equivalent to `{var=*}`. The syntax `LITERAL` matches literal
    text in the URL path. If the `LITERAL` contains any reserved character,
    such characters should be percent-encoded before the matching. If a
    variable contains exactly one path segment, such as `"{var}"` or
    `"{var=*}"`, when such a variable is expanded into a URL path on the client
    side, all characters except `[-_.~0-9a-zA-Z]` are percent-encoded. The
    server side does the reverse decoding. Such variables show up in the
    [Discovery
    Document](https://developers.google.com/discovery/v1/reference/apis) as
    `{var}`. If a variable contains multiple path segments, such as
    `"{var=foo/*}"` or `"{var=**}"`, when such a variable is expanded into a
    URL path on the client side, all characters except `[-_.~/0-9a-zA-Z]` are
    percent-encoded. The server side does the reverse decoding, except "%2F"
    and "%2f" are left unchanged. Such variables show up in the [Discovery
    Document](https://developers.google.com/discovery/v1/reference/apis) as
    `{+var}`. ## Using gRPC API Service Configuration gRPC API Service
    Configuration (service config) is a configuration language for configuring
    a gRPC service to become a user-facing product. The service config is
    simply the YAML representation of the `google.api.Service` proto message.
    As an alternative to annotating your proto file, you can configure gRPC
    transcoding in your service config YAML files. You do this by specifying a
    `HttpRule` that maps the gRPC method to a REST endpoint, achieving the same
    effect as the proto annotation. This can be particularly useful if you have
    a proto that is reused in multiple services. Note that any transcoding
    specified in the service config will override any matching transcoding
    configuration in the proto. Example:     http:       rules:         #
    Selects a gRPC method and applies HttpRule to it.         - selector:
    example.v1.Messaging.GetMessage           get:
    /v1/messages/{message_id}/{sub.subfield} ## Special notes When gRPC
    Transcoding is used to map a gRPC to JSON REST endpoints, the proto to JSON
    conversion must follow the [proto3
    specification](https://developers.google.com/protocol-
    buffers/docs/proto3#json). While the single segment variable follows the
    semantics of [RFC 6570](https://tools.ietf.org/html/rfc6570) Section 3.2.2
    Simple String Expansion, the multi segment variable **does not** follow RFC
    6570 Section 3.2.3 Reserved Expansion. The reason is that the Reserved
    Expansion does not expand special characters like `?` and `#`, which would
    lead to invalid URLs. As the result, gRPC Transcoding uses a custom
    encoding for multi segment variables. The path variables **must not** refer
    to any repeated or mapped field, because client libraries are not capable
    of handling such variable expansion. The path variables **must not**
    capture the leading "/" character. The reason is that the most common use
    case "{var}" does not capture the leading "/" character. For consistency,
    all path variables must share the same behavior. Repeated message fields
    must not be mapped to URL query parameters, because no client library can
    support such complicated mapping. If an API needs to use a JSON array for
    request or response body, it can map the request or response body to a
    repeated field. However, some gRPC Transcoding implementations may not
    support this feature.
    """

    selector: str = betterproto.string_field(1)
    """
    Selects a method to which this rule applies. Refer to
    [selector][google.api.DocumentationRule.selector] for syntax details.
    """

    get: str = betterproto.string_field(2, group="pattern")
    """
    Maps to HTTP GET. Used for listing and getting information about resources.
    """

    put: str = betterproto.string_field(3, group="pattern")
    """Maps to HTTP PUT. Used for replacing a resource."""

    post: str = betterproto.string_field(4, group="pattern")
    """
    Maps to HTTP POST. Used for creating a resource or performing an action.
    """

    delete: str = betterproto.string_field(5, group="pattern")
    """Maps to HTTP DELETE. Used for deleting a resource."""

    patch: str = betterproto.string_field(6, group="pattern")
    """Maps to HTTP PATCH. Used for updating a resource."""

    custom: "CustomHttpPattern" = betterproto.message_field(8, group="pattern")
    """
    The custom pattern is used for specifying an HTTP method that is not
    included in the `pattern` field, such as HEAD, or "*" to leave the HTTP
    method unspecified for this rule. The wild-card rule is useful for services
    that provide content to Web (HTML) clients.
    """

    body: str = betterproto.string_field(7)
    """
    The name of the request field whose value is mapped to the HTTP request
    body, or `*` for mapping all request fields not captured by the path
    pattern to the HTTP body, or omitted for not having any HTTP request body.
    NOTE: the referred field must be present at the top-level of the request
    message type.
    """

    response_body: str = betterproto.string_field(12)
    """
    Optional. The name of the response field whose value is mapped to the HTTP
    response body. When omitted, the entire response message will be used as
    the HTTP response body. NOTE: The referred field must be present at the
    top-level of the response message type.
    """

    additional_bindings: List["HttpRule"] = betterproto.message_field(11)
    """
    Additional HTTP bindings for the selector. Nested bindings must not contain
    an `additional_bindings` field themselves (that is, the nesting may only be
    one level deep).
    """


@dataclass(eq=False, repr=False)
class CustomHttpPattern(betterproto.Message):
    """A custom pattern is used for defining custom HTTP verb."""

    kind: str = betterproto.string_field(1)
    """The name of this custom HTTP verb."""

    path: str = betterproto.string_field(2)
    """The path matched by this custom verb."""



from unittest import mock

from pysisu.proto.sisu.v1.api import CreateDataSetRequestDataset
from .helpers_for_tests import SNAPSHOT_DICT


def validate_api_call(mock_call_sisu_api, method=None, path=None):
    assert mock_call_sisu_api.call_count == 1
    if method is not None:
        assert mock_call_sisu_api.call_args.kwargs["request_method"] == method
    if path is not None:
        assert mock_call_sisu_api.call_args[0][0] == path


@mock.patch("pysisu.pysisu_class.PySisu._call_sisu_api")
def test_run_analysis(mock_call_sisu_api, pysisu_client):
    pysisu_client.run(1)
    validate_api_call(
        mock_call_sisu_api, method="POST", path="api/v1/analyses/1/run"
    )


@mock.patch("pysisu.pysisu_class.PySisu._call_sisu_api")
def test_list_analyses(
    mock_call_sisu_api, analyses_list, pysisu_client, pysisu_input_snapshot
):
    mock_call_sisu_api.return_value = analyses_list.to_dict()
    pysisu_input_snapshot.assert_match(
        pysisu_client.analyses().to_json(indent=4),
        SNAPSHOT_DICT["list analyses"],
    )
    validate_api_call(mock_call_sisu_api, path="api/v1/analyses")


@mock.patch("pysisu.pysisu_class.PySisu._call_sisu_api")
def test_list_metrics(
    mock_call_sisu_api, metrics_list, pysisu_client, pysisu_input_snapshot
):
    mock_call_sisu_api.return_value = metrics_list.to_dict()
    pysisu_input_snapshot.assert_match(
        pysisu_client.metrics().to_json(indent=4),
        SNAPSHOT_DICT["list metrics"],
    )
    validate_api_call(mock_call_sisu_api, path="api/v1/metrics")


@mock.patch("pysisu.pysisu_class.PySisu._call_sisu_api")
def test_list_data_sources(
    mock_call_sisu_api, data_sources_list, pysisu_client, pysisu_input_snapshot
):
    mock_call_sisu_api.return_value = data_sources_list.to_dict()
    pysisu_input_snapshot.assert_match(
        pysisu_client.data_sources().to_json(indent=4),
        SNAPSHOT_DICT["list data sources"],
    )
    validate_api_call(mock_call_sisu_api, path="api/v1/data_sources")


@mock.patch("pysisu.pysisu_class.PySisu._call_sisu_api")
def test_list_datasets(
    mock_call_sisu_api, datasets_list, pysisu_client, pysisu_input_snapshot
):
    mock_call_sisu_api.return_value = datasets_list.to_dict()
    pysisu_input_snapshot.assert_match(
        pysisu_client.datasets().to_json(indent=4),
        SNAPSHOT_DICT["list datasets"],
    )
    validate_api_call(mock_call_sisu_api, path="api/v1/datasets")


@mock.patch("pysisu.pysisu_class.PySisu._call_sisu_api")
def test_get_analysis_filters(
    mock_call_sisu_api, analysis_filter, pysisu_client, pysisu_input_snapshot
):
    mock_call_sisu_api.return_value = analysis_filter.to_dict()
    pysisu_input_snapshot.assert_match(
        pysisu_client.get_filters(1).to_json(indent=4),
        SNAPSHOT_DICT["get analysis filters"],
    )
    validate_api_call(mock_call_sisu_api, path="api/v1/analyses/1/filters")


@mock.patch("pysisu.pysisu_class.PySisu._call_sisu_api")
def test_list_projects(
    mock_call_sisu_api, projects_list, pysisu_client, pysisu_input_snapshot
):
    mock_call_sisu_api.return_value = projects_list.to_dict()
    pysisu_input_snapshot.assert_match(
        pysisu_client.get_projects().to_json(indent=4),
        SNAPSHOT_DICT["list projects"],
    )
    validate_api_call(mock_call_sisu_api, "GET", "api/v1/projects")


@mock.patch("pysisu.pysisu_class.PySisu._call_sisu_api")
def test_list_project_analyses(
    mock_call_sisu_api,
    project_analyses_list,
    pysisu_client,
    pysisu_input_snapshot,
):
    mock_call_sisu_api.return_value = project_analyses_list.to_dict()
    pysisu_input_snapshot.assert_match(
        pysisu_client.get_project_analyses_list(1).to_json(indent=4),
        SNAPSHOT_DICT["list project analyses"],
    )
    validate_api_call(mock_call_sisu_api, "GET", "api/v1/projects/1/analyses")


@mock.patch("pysisu.pysisu_class.PySisu._call_sisu_api")
def test_set_filters(mock_call_sisu_api, pysisu_client):
    pysisu_client.set_filters(1, expr={})
    validate_api_call(mock_call_sisu_api, "PUT", "api/v1/analyses/1/filters")


@mock.patch("pysisu.pysisu_class.PySisu._call_sisu_api")
def test_duplicate_analysis(mock_call_sisu_api, pysisu_client):
    pysisu_client.duplicate_analysis(1)
    validate_api_call(
        mock_call_sisu_api, "POST", "api/v1/analyses/1/duplicate"
    )


@mock.patch("pysisu.pysisu_class.PySisu._call_sisu_api")
def test_get_factor_data(mock_call_sisu_api, pysisu_client):
    pysisu_client.get_factor_data(1)
    validate_api_call(
        mock_call_sisu_api, "GET", "api/v1/segments/1/data_query"
    )


@mock.patch("pysisu.pysisu_class.PySisu._call_sisu_api")
def test_create_dataset(mock_call_sisu_api, pysisu_client):
    pysisu_client.create_dataset(1, body=CreateDataSetRequestDataset())
    validate_api_call(
        mock_call_sisu_api, "POST", "api/v1/data_sources/1/datasets"
    )



from unittest import mock

from .helpers_for_tests import SNAPSHOT_DICT


@mock.patch("pysisu.PySisu.fetch_sisu_api")
def test_time_compare_proto(
    mock_pysisu, result_time_compare, pysisu_client, pysisu_input_snapshot
):
    mock_pysisu.return_value = result_time_compare.to_dict()
    pysisu_input_snapshot.assert_match(
        pysisu_client.get_results(1, format=None).to_json(indent=4),
        SNAPSHOT_DICT["time compare"],
    )


@mock.patch("pysisu.PySisu.fetch_sisu_api")
def test_group_compare_proto(
    mock_pysisu, result_group_compare, pysisu_client, pysisu_input_snapshot
):
    mock_pysisu.return_value = result_group_compare.to_dict()
    pysisu_input_snapshot.assert_match(
        pysisu_client.get_results(1, format=None).to_json(indent=4),
        SNAPSHOT_DICT["group compare"],
    )


@mock.patch("pysisu.PySisu.fetch_sisu_api")
def test_general_performance_proto(
    mock_pysisu,
    result_general_performance,
    pysisu_client,
    pysisu_input_snapshot,
):
    mock_pysisu.return_value = result_general_performance.to_dict()
    pysisu_input_snapshot.assert_match(
        pysisu_client.get_results(1, format=None).to_json(indent=4),
        SNAPSHOT_DICT["general performance"],
    )


@mock.patch("pysisu.PySisu.fetch_sisu_api")
def test_trend_detection_proto(
    mock_fetch, result_trend, pysisu_client, pysisu_input_snapshot
):
    mock_fetch.return_value = result_trend.to_dict()
    pysisu_input_snapshot.assert_match(
        pysisu_client.get_results(1, format=None).to_json(indent=4),
        SNAPSHOT_DICT["trend"],
    )



#
# Copyright 2022 Sisu Data, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from pathlib import Path

from setuptools import find_packages, setup

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()
pkg_vars = {}

# Doing versions is hard and annoying in python.
# See https://stackoverflow.com/questions/458550/standard-way-to-embed-version-into-python-package
with open("pysisu/version.py") as fp:
    exec(fp.read(), pkg_vars)

setup(
    name="pysisu",
    packages=find_packages(include=["pysisu", "pysisu.*"]),
    install_requires=["betterproto>=2.0.0b4", "requests-cache", "requests"],
    version=pkg_vars["__version__"],
    long_description=long_description,
    long_description_content_type="text/markdown",
)
